@@ -1,7 +1,10 @@
 import React from 'react';
 import ReactDOM from 'react-dom';
-import ReactFlightDOMClient from 'react-flight-dom-webpack';
+import ReactTransportDOMClient from 'react-transport-dom-webpack';
 import App from './App';
 
-let data = ReactFlightDOMClient.readFromFetch(fetch('http://localhost:3001'));
+let data = ReactTransportDOMClient.createFromFetch(
+  fetch('http://localhost:3001')
+);
+
 ReactDOM.render(<App data={data} />, document.getElementById('root'));
