@@ -7,9 +7,7 @@
  * @flow
  */
 
-import {addUserTimingListener} from 'shared/ReactFeatureFlags';
 import {isEnabled} from './src/events/ReactDOMEventListener';
-import {getClosestInstanceFromNode} from './src/client/ReactDOMComponentTree';
 
 import {__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED} from './src/client/ReactDOM';
 
@@ -18,11 +16,6 @@ Object.assign((__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED: any), {
   ReactBrowserEventEmitter: {
     isEnabled,
   },
-  ReactDOMComponentTree: {
-    getClosestInstanceFromNode,
-  },
-  // Perf experiment
-  addUserTimingListener,
 });
 
 export {
@@ -36,11 +29,14 @@ export {
   render,
   unmountComponentAtNode,
   createRoot,
+  createRoot as unstable_createRoot,
   createBlockingRoot,
-  unstable_discreteUpdates,
-  unstable_flushDiscreteUpdates,
+  createBlockingRoot as unstable_createBlockingRoot,
   unstable_flushControlled,
   unstable_scheduleHydration,
+  unstable_runWithPriority,
   unstable_renderSubtreeIntoContainer,
   unstable_createPortal,
+  unstable_createEventHandle,
+  unstable_isNewReconciler,
 } from './src/client/ReactDOM';
