@@ -7,15 +7,16 @@
  * @flow
  */
 
-import type {AnyNativeEvent} from 'legacy-events/PluginModuleType';
+import type {AnyNativeEvent} from '../events/PluginModuleType';
 import type {Container, SuspenseInstance} from '../client/ReactDOMHostConfig';
-import type {DOMTopLevelEventType} from 'legacy-events/TopLevelEventTypes';
-import type {EventSystemFlags} from 'legacy-events/EventSystemFlags';
-import type {FiberRoot} from 'react-reconciler/src/ReactFiberRoot';
+import type {DOMEventName} from '../events/DOMEventNames';
+import type {EventSystemFlags} from './EventSystemFlags';
+import type {FiberRoot} from 'react-reconciler/src/ReactInternalTypes';
+import type {LanePriority} from 'react-reconciler/src/ReactFiberLane';
 
 import {
-  enableDeprecatedFlareAPI,
   enableSelectiveHydration,
+  enableEagerRootListeners,
 } from 'shared/ReactFeatureFlags';
 import {
   unstable_runWithPriority as runWithPriority,
@@ -27,18 +28,13 @@ import {
   getNearestMountedFiber,
   getContainerFromFiber,
   getSuspenseInstanceFromFiber,
-} from 'react-reconciler/reflection';
-import {
-  attemptToDispatchEvent,
-  addResponderEventSystemEvent,
-} from './ReactDOMEventListener';
-import {getListenerMapForElement} from './DOMEventListenerMap';
+} from 'react-reconciler/src/ReactFiberTreeReflection';
+import {attemptToDispatchEvent} from './ReactDOMEventListener';
 import {
   getInstanceFromNode,
   getClosestInstanceFromNode,
 } from '../client/ReactDOMComponentTree';
-import {unsafeCastDOMTopLevelTypeToString} from 'legacy-events/TopLevelEventTypes';
-import {HostRoot, SuspenseComponent} from 'shared/ReactWorkTags';
+import {HostRoot, SuspenseComponent} from 'react-reconciler/src/ReactWorkTags';
 
 let attemptSynchronousHydration: (fiber: Object) => void;
 
@@ -66,6 +62,20 @@ export function setAttemptHydrationAtCurrentPriority(
   attemptHydrationAtCurrentPriority = fn;
 }
 
+let getCurrentUpdatePriority: () => LanePriority;
+
+export function setGetCurrentUpdatePriority(fn: () => LanePriority) {
+  getCurrentUpdatePriority = fn;
+}
+
+let attemptHydrationAtPriority: <T>(priority: LanePriority, fn: () => T) => T;
+
+export function setAttemptHydrationAtPriority(
+  fn: <T>(priority: LanePriority, fn: () => T) => T,
+) {
+  attemptHydrationAtPriority = fn;
+}
+
 // TODO: Upgrade this definition once we're on a newer version of Flow that
 // has this definition built-in.
 type PointerEvent = Event & {
@@ -74,82 +84,41 @@ type PointerEvent = Event & {
   ...
 };
 
-import {
-  TOP_MOUSE_DOWN,
-  TOP_MOUSE_UP,
-  TOP_TOUCH_CANCEL,
-  TOP_TOUCH_END,
-  TOP_TOUCH_START,
-  TOP_AUX_CLICK,
-  TOP_DOUBLE_CLICK,
-  TOP_POINTER_CANCEL,
-  TOP_POINTER_DOWN,
-  TOP_POINTER_UP,
-  TOP_DRAG_END,
-  TOP_DRAG_START,
-  TOP_DROP,
-  TOP_COMPOSITION_END,
-  TOP_COMPOSITION_START,
-  TOP_KEY_DOWN,
-  TOP_KEY_PRESS,
-  TOP_KEY_UP,
-  TOP_INPUT,
-  TOP_TEXT_INPUT,
-  TOP_CLOSE,
-  TOP_CANCEL,
-  TOP_COPY,
-  TOP_CUT,
-  TOP_PASTE,
-  TOP_CLICK,
-  TOP_CHANGE,
-  TOP_CONTEXT_MENU,
-  TOP_RESET,
-  TOP_SUBMIT,
-  TOP_DRAG_ENTER,
-  TOP_DRAG_LEAVE,
-  TOP_MOUSE_OVER,
-  TOP_MOUSE_OUT,
-  TOP_POINTER_OVER,
-  TOP_POINTER_OUT,
-  TOP_GOT_POINTER_CAPTURE,
-  TOP_LOST_POINTER_CAPTURE,
-  TOP_FOCUS,
-  TOP_BLUR,
-} from './DOMTopLevelEventTypes';
-import {IS_REPLAYED} from 'legacy-events/EventSystemFlags';
-import {legacyListenToTopLevelEvent} from './DOMLegacyEventPluginSystem';
+import {IS_REPLAYED} from './EventSystemFlags';
+import {listenToNativeEvent} from './DOMPluginEventSystem';
 
 type QueuedReplayableEvent = {|
   blockedOn: null | Container | SuspenseInstance,
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   eventSystemFlags: EventSystemFlags,
   nativeEvent: AnyNativeEvent,
-  container: Document | Element | Node,
+  targetContainers: Array<EventTarget>,
 |};
 
 let hasScheduledReplayAttempt = false;
 
 // The queue of discrete events to be replayed.
-let queuedDiscreteEvents: Array<QueuedReplayableEvent> = [];
+const queuedDiscreteEvents: Array<QueuedReplayableEvent> = [];
 
 // Indicates if any continuous event targets are non-null for early bailout.
-let hasAnyQueuedContinuousEvents: boolean = false;
+const hasAnyQueuedContinuousEvents: boolean = false;
 // The last of each continuous event type. We only need to replay the last one
 // if the last target was dehydrated.
 let queuedFocus: null | QueuedReplayableEvent = null;
 let queuedDrag: null | QueuedReplayableEvent = null;
 let queuedMouse: null | QueuedReplayableEvent = null;
 // For pointer events there can be one latest event per pointerId.
-let queuedPointers: Map<number, QueuedReplayableEvent> = new Map();
-let queuedPointerCaptures: Map<number, QueuedReplayableEvent> = new Map();
+const queuedPointers: Map<number, QueuedReplayableEvent> = new Map();
+const queuedPointerCaptures: Map<number, QueuedReplayableEvent> = new Map();
 // We could consider replaying selectionchange and touchmoves too.
 
 type QueuedHydrationTarget = {|
   blockedOn: null | Container | SuspenseInstance,
   target: Node,
   priority: number,
+  lanePriority: LanePriority,
 |};
-let queuedExplicitHydrationTargets: Array<QueuedHydrationTarget> = [];
+const queuedExplicitHydrationTargets: Array<QueuedHydrationTarget> = [];
 
 export function hasQueuedDiscreteEvents(): boolean {
   return queuedDiscreteEvents.length > 0;
@@ -159,80 +128,61 @@ export function hasQueuedContinuousEvents(): boolean {
   return hasAnyQueuedContinuousEvents;
 }
 
-const discreteReplayableEvents = [
-  TOP_MOUSE_DOWN,
-  TOP_MOUSE_UP,
-  TOP_TOUCH_CANCEL,
-  TOP_TOUCH_END,
-  TOP_TOUCH_START,
-  TOP_AUX_CLICK,
-  TOP_DOUBLE_CLICK,
-  TOP_POINTER_CANCEL,
-  TOP_POINTER_DOWN,
-  TOP_POINTER_UP,
-  TOP_DRAG_END,
-  TOP_DRAG_START,
-  TOP_DROP,
-  TOP_COMPOSITION_END,
-  TOP_COMPOSITION_START,
-  TOP_KEY_DOWN,
-  TOP_KEY_PRESS,
-  TOP_KEY_UP,
-  TOP_INPUT,
-  TOP_TEXT_INPUT,
-  TOP_CLOSE,
-  TOP_CANCEL,
-  TOP_COPY,
-  TOP_CUT,
-  TOP_PASTE,
-  TOP_CLICK,
-  TOP_CHANGE,
-  TOP_CONTEXT_MENU,
-  TOP_RESET,
-  TOP_SUBMIT,
+const discreteReplayableEvents: Array<DOMEventName> = [
+  'mousedown',
+  'mouseup',
+  'touchcancel',
+  'touchend',
+  'touchstart',
+  'auxclick',
+  'dblclick',
+  'pointercancel',
+  'pointerdown',
+  'pointerup',
+  'dragend',
+  'dragstart',
+  'drop',
+  'compositionend',
+  'compositionstart',
+  'keydown',
+  'keypress',
+  'keyup',
+  'input',
+  'textInput', // Intentionally camelCase
+  'copy',
+  'cut',
+  'paste',
+  'click',
+  'change',
+  'contextmenu',
+  'reset',
+  'submit',
 ];
 
-const continuousReplayableEvents = [
-  TOP_FOCUS,
-  TOP_BLUR,
-  TOP_DRAG_ENTER,
-  TOP_DRAG_LEAVE,
-  TOP_MOUSE_OVER,
-  TOP_MOUSE_OUT,
-  TOP_POINTER_OVER,
-  TOP_POINTER_OUT,
-  TOP_GOT_POINTER_CAPTURE,
-  TOP_LOST_POINTER_CAPTURE,
+const continuousReplayableEvents: Array<DOMEventName> = [
+  'dragenter',
+  'dragleave',
+  'focusin',
+  'focusout',
+  'mouseover',
+  'mouseout',
+  'pointerover',
+  'pointerout',
+  'gotpointercapture',
+  'lostpointercapture',
 ];
 
-export function isReplayableDiscreteEvent(
-  eventType: DOMTopLevelEventType,
-): boolean {
+export function isReplayableDiscreteEvent(eventType: DOMEventName): boolean {
   return discreteReplayableEvents.indexOf(eventType) > -1;
 }
 
-function trapReplayableEventForDocument(
-  topLevelType: DOMTopLevelEventType,
-  document: Document,
-  listenerMap: Map<DOMTopLevelEventType | string, null | (any => void)>,
+function trapReplayableEventForContainer(
+  domEventName: DOMEventName,
+  container: Container,
 ) {
-  legacyListenToTopLevelEvent(topLevelType, document, listenerMap);
-  if (enableDeprecatedFlareAPI) {
-    // Trap events for the responder system.
-    const topLevelTypeString = unsafeCastDOMTopLevelTypeToString(topLevelType);
-    // TODO: Ideally we shouldn't need these to be active but
-    // if we only have a passive listener, we at least need it
-    // to still pretend to be active so that Flare gets those
-    // events.
-    const activeEventKey = topLevelTypeString + '_active';
-    if (!listenerMap.has(activeEventKey)) {
-      const listener = addResponderEventSystemEvent(
-        document,
-        topLevelTypeString,
-        false,
-      );
-      listenerMap.set(activeEventKey, listener);
-    }
+  // When the flag is on, we do this in a unified codepath elsewhere.
+  if (!enableEagerRootListeners) {
+    listenToNativeEvent(domEventName, false, ((container: any): Element), null);
   }
 }
 
@@ -240,45 +190,47 @@ export function eagerlyTrapReplayableEvents(
   container: Container,
   document: Document,
 ) {
-  const listenerMapForDoc = getListenerMapForElement(document);
-  // Discrete
-  discreteReplayableEvents.forEach(topLevelType => {
-    trapReplayableEventForDocument(topLevelType, document, listenerMapForDoc);
-  });
-  // Continuous
-  continuousReplayableEvents.forEach(topLevelType => {
-    trapReplayableEventForDocument(topLevelType, document, listenerMapForDoc);
-  });
+  // When the flag is on, we do this in a unified codepath elsewhere.
+  if (!enableEagerRootListeners) {
+    // Discrete
+    discreteReplayableEvents.forEach(domEventName => {
+      trapReplayableEventForContainer(domEventName, container);
+    });
+    // Continuous
+    continuousReplayableEvents.forEach(domEventName => {
+      trapReplayableEventForContainer(domEventName, container);
+    });
+  }
 }
 
 function createQueuedReplayableEvent(
   blockedOn: null | Container | SuspenseInstance,
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   eventSystemFlags: EventSystemFlags,
-  container: Document | Element | Node,
+  targetContainer: EventTarget,
   nativeEvent: AnyNativeEvent,
 ): QueuedReplayableEvent {
   return {
     blockedOn,
-    topLevelType,
+    domEventName,
     eventSystemFlags: eventSystemFlags | IS_REPLAYED,
     nativeEvent,
-    container,
+    targetContainers: [targetContainer],
   };
 }
 
 export function queueDiscreteEvent(
   blockedOn: null | Container | SuspenseInstance,
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   eventSystemFlags: EventSystemFlags,
-  container: Document | Element | Node,
+  targetContainer: EventTarget,
   nativeEvent: AnyNativeEvent,
 ): void {
   const queuedEvent = createQueuedReplayableEvent(
     blockedOn,
-    topLevelType,
+    domEventName,
     eventSystemFlags,
-    container,
+    targetContainer,
     nativeEvent,
   );
   queuedDiscreteEvents.push(queuedEvent);
@@ -287,7 +239,7 @@ export function queueDiscreteEvent(
       // If this was the first discrete event, we might be able to
       // synchronously unblock it so that preventDefault still works.
       while (queuedEvent.blockedOn !== null) {
-        let fiber = getInstanceFromNode(queuedEvent.blockedOn);
+        const fiber = getInstanceFromNode(queuedEvent.blockedOn);
         if (fiber === null) {
           break;
         }
@@ -310,31 +262,31 @@ export function queueDiscreteEvent(
 
 // Resets the replaying for this type of continuous event to no event.
 export function clearIfContinuousEvent(
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   nativeEvent: AnyNativeEvent,
 ): void {
-  switch (topLevelType) {
-    case TOP_FOCUS:
-    case TOP_BLUR:
+  switch (domEventName) {
+    case 'focusin':
+    case 'focusout':
       queuedFocus = null;
       break;
-    case TOP_DRAG_ENTER:
-    case TOP_DRAG_LEAVE:
+    case 'dragenter':
+    case 'dragleave':
       queuedDrag = null;
       break;
-    case TOP_MOUSE_OVER:
-    case TOP_MOUSE_OUT:
+    case 'mouseover':
+    case 'mouseout':
       queuedMouse = null;
       break;
-    case TOP_POINTER_OVER:
-    case TOP_POINTER_OUT: {
-      let pointerId = ((nativeEvent: any): PointerEvent).pointerId;
+    case 'pointerover':
+    case 'pointerout': {
+      const pointerId = ((nativeEvent: any): PointerEvent).pointerId;
       queuedPointers.delete(pointerId);
       break;
     }
-    case TOP_GOT_POINTER_CAPTURE:
-    case TOP_LOST_POINTER_CAPTURE: {
-      let pointerId = ((nativeEvent: any): PointerEvent).pointerId;
+    case 'gotpointercapture':
+    case 'lostpointercapture': {
+      const pointerId = ((nativeEvent: any): PointerEvent).pointerId;
       queuedPointerCaptures.delete(pointerId);
       break;
     }
@@ -344,24 +296,24 @@ export function clearIfContinuousEvent(
 function accumulateOrCreateContinuousQueuedReplayableEvent(
   existingQueuedEvent: null | QueuedReplayableEvent,
   blockedOn: null | Container | SuspenseInstance,
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   eventSystemFlags: EventSystemFlags,
-  container: Document | Element | Node,
+  targetContainer: EventTarget,
   nativeEvent: AnyNativeEvent,
 ): QueuedReplayableEvent {
   if (
     existingQueuedEvent === null ||
     existingQueuedEvent.nativeEvent !== nativeEvent
   ) {
-    let queuedEvent = createQueuedReplayableEvent(
+    const queuedEvent = createQueuedReplayableEvent(
       blockedOn,
-      topLevelType,
+      domEventName,
       eventSystemFlags,
-      container,
+      targetContainer,
       nativeEvent,
     );
     if (blockedOn !== null) {
-      let fiber = getInstanceFromNode(blockedOn);
+      const fiber = getInstanceFromNode(blockedOn);
       if (fiber !== null) {
         // Attempt to increase the priority of this target.
         attemptContinuousHydration(fiber);
@@ -371,60 +323,67 @@ function accumulateOrCreateContinuousQueuedReplayableEvent(
   }
   // If we have already queued this exact event, then it's because
   // the different event systems have different DOM event listeners.
-  // We can accumulate the flags and store a single event to be
-  // replayed.
+  // We can accumulate the flags, and the targetContainers, and
+  // store a single event to be replayed.
   existingQueuedEvent.eventSystemFlags |= eventSystemFlags;
+  const targetContainers = existingQueuedEvent.targetContainers;
+  if (
+    targetContainer !== null &&
+    targetContainers.indexOf(targetContainer) === -1
+  ) {
+    targetContainers.push(targetContainer);
+  }
   return existingQueuedEvent;
 }
 
 export function queueIfContinuousEvent(
   blockedOn: null | Container | SuspenseInstance,
-  topLevelType: DOMTopLevelEventType,
+  domEventName: DOMEventName,
   eventSystemFlags: EventSystemFlags,
-  container: Document | Element | Node,
+  targetContainer: EventTarget,
   nativeEvent: AnyNativeEvent,
 ): boolean {
   // These set relatedTarget to null because the replayed event will be treated as if we
   // moved from outside the window (no target) onto the target once it hydrates.
   // Instead of mutating we could clone the event.
-  switch (topLevelType) {
-    case TOP_FOCUS: {
+  switch (domEventName) {
+    case 'focusin': {
       const focusEvent = ((nativeEvent: any): FocusEvent);
       queuedFocus = accumulateOrCreateContinuousQueuedReplayableEvent(
         queuedFocus,
         blockedOn,
-        topLevelType,
+        domEventName,
         eventSystemFlags,
-        container,
+        targetContainer,
         focusEvent,
       );
       return true;
     }
-    case TOP_DRAG_ENTER: {
+    case 'dragenter': {
       const dragEvent = ((nativeEvent: any): DragEvent);
       queuedDrag = accumulateOrCreateContinuousQueuedReplayableEvent(
         queuedDrag,
         blockedOn,
-        topLevelType,
+        domEventName,
         eventSystemFlags,
-        container,
+        targetContainer,
         dragEvent,
       );
       return true;
     }
-    case TOP_MOUSE_OVER: {
+    case 'mouseover': {
       const mouseEvent = ((nativeEvent: any): MouseEvent);
       queuedMouse = accumulateOrCreateContinuousQueuedReplayableEvent(
         queuedMouse,
         blockedOn,
-        topLevelType,
+        domEventName,
         eventSystemFlags,
-        container,
+        targetContainer,
         mouseEvent,
       );
       return true;
     }
-    case TOP_POINTER_OVER: {
+    case 'pointerover': {
       const pointerEvent = ((nativeEvent: any): PointerEvent);
       const pointerId = pointerEvent.pointerId;
       queuedPointers.set(
@@ -432,15 +391,15 @@ export function queueIfContinuousEvent(
         accumulateOrCreateContinuousQueuedReplayableEvent(
           queuedPointers.get(pointerId) || null,
           blockedOn,
-          topLevelType,
+          domEventName,
           eventSystemFlags,
-          container,
+          targetContainer,
           pointerEvent,
         ),
       );
       return true;
     }
-    case TOP_GOT_POINTER_CAPTURE: {
+    case 'gotpointercapture': {
       const pointerEvent = ((nativeEvent: any): PointerEvent);
       const pointerId = pointerEvent.pointerId;
       queuedPointerCaptures.set(
@@ -448,9 +407,9 @@ export function queueIfContinuousEvent(
         accumulateOrCreateContinuousQueuedReplayableEvent(
           queuedPointerCaptures.get(pointerId) || null,
           blockedOn,
-          topLevelType,
+          domEventName,
           eventSystemFlags,
-          container,
+          targetContainer,
           pointerEvent,
         ),
       );
@@ -467,20 +426,23 @@ function attemptExplicitHydrationTarget(
   // TODO: This function shares a lot of logic with attemptToDispatchEvent.
   // Try to unify them. It's a bit tricky since it would require two return
   // values.
-  let targetInst = getClosestInstanceFromNode(queuedTarget.target);
+  const targetInst = getClosestInstanceFromNode(queuedTarget.target);
   if (targetInst !== null) {
-    let nearestMounted = getNearestMountedFiber(targetInst);
+    const nearestMounted = getNearestMountedFiber(targetInst);
     if (nearestMounted !== null) {
       const tag = nearestMounted.tag;
       if (tag === SuspenseComponent) {
-        let instance = getSuspenseInstanceFromFiber(nearestMounted);
+        const instance = getSuspenseInstanceFromFiber(nearestMounted);
         if (instance !== null) {
           // We're blocked on hydrating this boundary.
           // Increase its priority.
           queuedTarget.blockedOn = instance;
-          runWithPriority(queuedTarget.priority, () => {
-            attemptHydrationAtCurrentPriority(nearestMounted);
+          attemptHydrationAtPriority(queuedTarget.lanePriority, () => {
+            runWithPriority(queuedTarget.priority, () => {
+              attemptHydrationAtCurrentPriority(nearestMounted);
+            });
           });
+
           return;
         }
       } else if (tag === HostRoot) {
@@ -499,15 +461,17 @@ function attemptExplicitHydrationTarget(
 
 export function queueExplicitHydrationTarget(target: Node): void {
   if (enableSelectiveHydration) {
-    let priority = getCurrentPriorityLevel();
+    const schedulerPriority = getCurrentPriorityLevel();
+    const updateLanePriority = getCurrentUpdatePriority();
     const queuedTarget: QueuedHydrationTarget = {
       blockedOn: null,
       target: target,
-      priority: priority,
+      priority: schedulerPriority,
+      lanePriority: updateLanePriority,
     };
     let i = 0;
     for (; i < queuedExplicitHydrationTargets.length; i++) {
-      if (priority <= queuedExplicitHydrationTargets[i].priority) {
+      if (schedulerPriority <= queuedExplicitHydrationTargets[i].priority) {
         break;
       }
     }
@@ -524,20 +488,26 @@ function attemptReplayContinuousQueuedEvent(
   if (queuedEvent.blockedOn !== null) {
     return false;
   }
-  let nextBlockedOn = attemptToDispatchEvent(
-    queuedEvent.topLevelType,
-    queuedEvent.eventSystemFlags,
-    queuedEvent.container,
-    queuedEvent.nativeEvent,
-  );
-  if (nextBlockedOn !== null) {
-    // We're still blocked. Try again later.
-    let fiber = getInstanceFromNode(nextBlockedOn);
-    if (fiber !== null) {
-      attemptContinuousHydration(fiber);
+  const targetContainers = queuedEvent.targetContainers;
+  while (targetContainers.length > 0) {
+    const targetContainer = targetContainers[0];
+    const nextBlockedOn = attemptToDispatchEvent(
+      queuedEvent.domEventName,
+      queuedEvent.eventSystemFlags,
+      targetContainer,
+      queuedEvent.nativeEvent,
+    );
+    if (nextBlockedOn !== null) {
+      // We're still blocked. Try again later.
+      const fiber = getInstanceFromNode(nextBlockedOn);
+      if (fiber !== null) {
+        attemptContinuousHydration(fiber);
+      }
+      queuedEvent.blockedOn = nextBlockedOn;
+      return false;
     }
-    queuedEvent.blockedOn = nextBlockedOn;
-    return false;
+    // This target container was successfully dispatched. Try the next.
+    targetContainers.shift();
   }
   return true;
 }
@@ -556,27 +526,35 @@ function replayUnblockedEvents() {
   hasScheduledReplayAttempt = false;
   // First replay discrete events.
   while (queuedDiscreteEvents.length > 0) {
-    let nextDiscreteEvent = queuedDiscreteEvents[0];
+    const nextDiscreteEvent = queuedDiscreteEvents[0];
     if (nextDiscreteEvent.blockedOn !== null) {
       // We're still blocked.
       // Increase the priority of this boundary to unblock
       // the next discrete event.
-      let fiber = getInstanceFromNode(nextDiscreteEvent.blockedOn);
+      const fiber = getInstanceFromNode(nextDiscreteEvent.blockedOn);
       if (fiber !== null) {
         attemptUserBlockingHydration(fiber);
       }
       break;
     }
-    let nextBlockedOn = attemptToDispatchEvent(
-      nextDiscreteEvent.topLevelType,
-      nextDiscreteEvent.eventSystemFlags,
-      nextDiscreteEvent.container,
-      nextDiscreteEvent.nativeEvent,
-    );
-    if (nextBlockedOn !== null) {
-      // We're still blocked. Try again later.
-      nextDiscreteEvent.blockedOn = nextBlockedOn;
-    } else {
+    const targetContainers = nextDiscreteEvent.targetContainers;
+    while (targetContainers.length > 0) {
+      const targetContainer = targetContainers[0];
+      const nextBlockedOn = attemptToDispatchEvent(
+        nextDiscreteEvent.domEventName,
+        nextDiscreteEvent.eventSystemFlags,
+        targetContainer,
+        nextDiscreteEvent.nativeEvent,
+      );
+      if (nextBlockedOn !== null) {
+        // We're still blocked. Try again later.
+        nextDiscreteEvent.blockedOn = nextBlockedOn;
+        break;
+      }
+      // This target container was successfully dispatched. Try the next.
+      targetContainers.shift();
+    }
+    if (nextDiscreteEvent.blockedOn === null) {
       // We've successfully replayed the first event. Let's try the next one.
       queuedDiscreteEvents.shift();
     }
@@ -622,7 +600,7 @@ export function retryIfBlockedOn(
     // worth it because we expect very few discrete events to queue up and once
     // we are actually fully unblocked it will be fast to replay them.
     for (let i = 1; i < queuedDiscreteEvents.length; i++) {
-      let queuedEvent = queuedDiscreteEvents[i];
+      const queuedEvent = queuedDiscreteEvents[i];
       if (queuedEvent.blockedOn === unblocked) {
         queuedEvent.blockedOn = null;
       }
@@ -644,14 +622,14 @@ export function retryIfBlockedOn(
   queuedPointerCaptures.forEach(unblock);
 
   for (let i = 0; i < queuedExplicitHydrationTargets.length; i++) {
-    let queuedTarget = queuedExplicitHydrationTargets[i];
+    const queuedTarget = queuedExplicitHydrationTargets[i];
     if (queuedTarget.blockedOn === unblocked) {
       queuedTarget.blockedOn = null;
     }
   }
 
   while (queuedExplicitHydrationTargets.length > 0) {
-    let nextExplicitTarget = queuedExplicitHydrationTargets[0];
+    const nextExplicitTarget = queuedExplicitHydrationTargets[0];
     if (nextExplicitTarget.blockedOn !== null) {
       // We're still blocked.
       break;
