@@ -10,8 +10,8 @@
 
 'use strict';
 
+let ReactFeatureFlags = require('shared/ReactFeatureFlags');
 let PropTypes;
-let ReactFeatureFlags;
 let React;
 let ReactNoop;
 let Scheduler;
@@ -20,7 +20,6 @@ describe('ReactIncrementalErrorHandling', () => {
   beforeEach(() => {
     jest.resetModules();
     ReactFeatureFlags = require('shared/ReactFeatureFlags');
-    ReactFeatureFlags.debugRenderPhaseSideEffectsForStrictMode = false;
     ReactFeatureFlags.replayFailedUnitOfWorkWithInvokeGuardedCallback = false;
     PropTypes = require('prop-types');
     React = require('react');
@@ -38,7 +37,25 @@ describe('ReactIncrementalErrorHandling', () => {
   }
 
   function normalizeCodeLocInfo(str) {
-    return str && str.replace(/\(at .+?:\d+\)/g, '(at **)');
+    return (
+      str &&
+      str.replace(/\n +(?:at|in) ([\S]+)[^\n]*/g, function(m, name) {
+        return '\n    in ' + name + ' (at **)';
+      })
+    );
+  }
+
+  // Note: This is based on a similar component we use in www. We can delete
+  // once the extra div wrapper is no longer necessary.
+  function LegacyHiddenDiv({children, mode}) {
+    return (
+      <div hidden={mode === 'hidden'}>
+        <React.unstable_LegacyHidden
+          mode={mode === 'hidden' ? 'unstable-defer-without-hiding' : mode}>
+          {children}
+        </React.unstable_LegacyHidden>
+      </div>
+    );
   }
 
   it('recovers from errors asynchronously', () => {
@@ -58,17 +75,17 @@ describe('ReactIncrementalErrorHandling', () => {
       }
     }
 
-    function ErrorMessage(props) {
+    function ErrorMessage({error}) {
       Scheduler.unstable_yieldValue('ErrorMessage');
-      return <span prop={`Caught an error: ${props.error.message}`} />;
+      return <span prop={`Caught an error: ${error.message}`} />;
     }
 
-    function Indirection(props) {
+    function Indirection({children}) {
       Scheduler.unstable_yieldValue('Indirection');
-      return props.children || null;
+      return children || null;
     }
 
-    function BadRender() {
+    function BadRender({unused}) {
       Scheduler.unstable_yieldValue('throw');
       throw new Error('oops!');
     }
@@ -152,17 +169,17 @@ describe('ReactIncrementalErrorHandling', () => {
       }
     }
 
-    function ErrorMessage(props) {
+    function ErrorMessage({error}) {
       Scheduler.unstable_yieldValue('ErrorMessage');
-      return <span prop={`Caught an error: ${props.error.message}`} />;
+      return <span prop={`Caught an error: ${error.message}`} />;
     }
 
-    function Indirection(props) {
+    function Indirection({children}) {
       Scheduler.unstable_yieldValue('Indirection');
-      return props.children || null;
+      return children || null;
     }
 
-    function BadRender() {
+    function BadRender({unused}) {
       Scheduler.unstable_yieldValue('throw');
       throw new Error('oops!');
     }
@@ -235,8 +252,9 @@ describe('ReactIncrementalErrorHandling', () => {
       });
     }
 
-    ReactNoop.render(<App isBroken={true} />, onCommit);
-    Scheduler.unstable_advanceTime(1000);
+    ReactNoop.discreteUpdates(() => {
+      ReactNoop.render(<App isBroken={true} />, onCommit);
+    });
     expect(Scheduler).toFlushAndYieldThrough(['error']);
     interrupt();
 
@@ -265,6 +283,7 @@ describe('ReactIncrementalErrorHandling', () => {
     expect(ReactNoop.getChildren()).toEqual([span('Everything is fine.')]);
   });
 
+  // @gate experimental
   it('does not include offscreen work when retrying after an error', () => {
     function App(props) {
       if (props.isBroken) {
@@ -275,9 +294,9 @@ describe('ReactIncrementalErrorHandling', () => {
       return (
         <>
           Everything is fine
-          <div hidden={true}>
+          <LegacyHiddenDiv mode="hidden">
             <div>Offscreen content</div>
-          </div>
+          </LegacyHiddenDiv>
         </>
       );
     }
@@ -292,8 +311,9 @@ describe('ReactIncrementalErrorHandling', () => {
       });
     }
 
-    ReactNoop.render(<App isBroken={true} />, onCommit);
-    Scheduler.unstable_advanceTime(1000);
+    ReactNoop.discreteUpdates(() => {
+      ReactNoop.render(<App isBroken={true} />, onCommit);
+    });
     expect(Scheduler).toFlushAndYieldThrough(['error']);
     interrupt();
 
@@ -342,17 +362,17 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('retries one more time before handling error', () => {
-    function BadRender() {
+    function BadRender({unused}) {
       Scheduler.unstable_yieldValue('BadRender');
       throw new Error('oops');
     }
 
-    function Sibling() {
+    function Sibling({unused}) {
       Scheduler.unstable_yieldValue('Sibling');
       return <span prop="Sibling" />;
     }
 
-    function Parent() {
+    function Parent({unused}) {
       Scheduler.unstable_yieldValue('Parent');
       return (
         <>
@@ -381,6 +401,56 @@ describe('ReactIncrementalErrorHandling', () => {
     expect(ReactNoop.getChildren()).toEqual([]);
   });
 
+  it('retries one more time if an error occurs during a render that expires midway through the tree', () => {
+    function Oops({unused}) {
+      Scheduler.unstable_yieldValue('Oops');
+      throw new Error('Oops');
+    }
+
+    function Text({text}) {
+      Scheduler.unstable_yieldValue(text);
+      return text;
+    }
+
+    function App({unused}) {
+      return (
+        <>
+          <Text text="A" />
+          <Text text="B" />
+          <Oops />
+          <Text text="C" />
+          <Text text="D" />
+        </>
+      );
+    }
+
+    ReactNoop.render(<App />);
+
+    // Render part of the tree
+    expect(Scheduler).toFlushAndYieldThrough(['A', 'B']);
+
+    // Expire the render midway through
+    Scheduler.unstable_advanceTime(10000);
+    expect(() => Scheduler.unstable_flushExpired()).toThrow('Oops');
+
+    expect(Scheduler).toHaveYielded([
+      // The render expired, but we shouldn't throw out the partial work.
+      // Finish the current level.
+      'Oops',
+      'C',
+      'D',
+
+      // Since the error occurred during a partially concurrent render, we should
+      // retry one more time, synchronously.
+      'A',
+      'B',
+      'Oops',
+      'C',
+      'D',
+    ]);
+    expect(ReactNoop.getChildren()).toEqual([]);
+  });
+
   it('calls componentDidCatch multiple times for multiple errors', () => {
     let id = 0;
     class BadMount extends React.Component {
@@ -476,7 +546,7 @@ describe('ReactIncrementalErrorHandling', () => {
       }
     }
 
-    function BrokenRender(props) {
+    function BrokenRender({unused}) {
       Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
@@ -504,27 +574,26 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('catches render error in a boundary during synchronous mounting', () => {
-    const ops = [];
     class ErrorBoundary extends React.Component {
       state = {error: null};
       componentDidCatch(error) {
-        ops.push('ErrorBoundary componentDidCatch');
+        Scheduler.unstable_yieldValue('ErrorBoundary componentDidCatch');
         this.setState({error});
       }
       render() {
         if (this.state.error) {
-          ops.push('ErrorBoundary render error');
+          Scheduler.unstable_yieldValue('ErrorBoundary render error');
           return (
             <span prop={`Caught an error: ${this.state.error.message}.`} />
           );
         }
-        ops.push('ErrorBoundary render success');
+        Scheduler.unstable_yieldValue('ErrorBoundary render success');
         return this.props.children;
       }
     }
 
-    function BrokenRender(props) {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
@@ -536,10 +605,15 @@ describe('ReactIncrementalErrorHandling', () => {
       );
     });
 
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'ErrorBoundary render success',
       'BrokenRender',
-      // React doesn't retry because we're already rendering synchronously.
+
+      // React retries one more time
+      'ErrorBoundary render success',
+      'BrokenRender',
+
+      // Errored again on retry. Now handle it.
       'ErrorBoundary componentDidCatch',
       'ErrorBoundary render error',
     ]);
@@ -547,27 +621,26 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('catches render error in a boundary during batched mounting', () => {
-    const ops = [];
     class ErrorBoundary extends React.Component {
       state = {error: null};
       componentDidCatch(error) {
-        ops.push('ErrorBoundary componentDidCatch');
+        Scheduler.unstable_yieldValue('ErrorBoundary componentDidCatch');
         this.setState({error});
       }
       render() {
         if (this.state.error) {
-          ops.push('ErrorBoundary render error');
+          Scheduler.unstable_yieldValue('ErrorBoundary render error');
           return (
             <span prop={`Caught an error: ${this.state.error.message}.`} />
           );
         }
-        ops.push('ErrorBoundary render success');
+        Scheduler.unstable_yieldValue('ErrorBoundary render success');
         return this.props.children;
       }
     }
 
-    function BrokenRender(props) {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
@@ -580,10 +653,15 @@ describe('ReactIncrementalErrorHandling', () => {
       );
     });
 
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'ErrorBoundary render success',
       'BrokenRender',
-      // React doesn't retry because we're already rendering synchronously.
+
+      // React retries one more time
+      'ErrorBoundary render success',
+      'BrokenRender',
+
+      // Errored again on retry. Now handle it.
       'ErrorBoundary componentDidCatch',
       'ErrorBoundary render error',
     ]);
@@ -591,20 +669,19 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('propagates an error from a noop error boundary during full deferred mounting', () => {
-    const ops = [];
     class RethrowErrorBoundary extends React.Component {
       componentDidCatch(error) {
-        ops.push('RethrowErrorBoundary componentDidCatch');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary componentDidCatch');
         throw error;
       }
       render() {
-        ops.push('RethrowErrorBoundary render');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary render');
         return this.props.children;
       }
     }
 
-    function BrokenRender() {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
@@ -615,19 +692,18 @@ describe('ReactIncrementalErrorHandling', () => {
     );
 
     expect(() => {
-      expect(Scheduler).toFlushWithoutYielding();
+      expect(Scheduler).toFlushAndYield([
+        'RethrowErrorBoundary render',
+        'BrokenRender',
+
+        // React retries one more time
+        'RethrowErrorBoundary render',
+        'BrokenRender',
+
+        // Errored again on retry. Now handle it.
+        'RethrowErrorBoundary componentDidCatch',
+      ]);
     }).toThrow('Hello');
-    expect(ops).toEqual([
-      'RethrowErrorBoundary render',
-      'BrokenRender',
-
-      // React retries one more time
-      'RethrowErrorBoundary render',
-      'BrokenRender',
-
-      // Errored again on retry. Now handle it.
-      'RethrowErrorBoundary componentDidCatch',
-    ]);
     expect(ReactNoop.getChildren()).toEqual([]);
   });
 
@@ -643,7 +719,7 @@ describe('ReactIncrementalErrorHandling', () => {
       }
     }
 
-    function BrokenRender() {
+    function BrokenRender({unused}) {
       Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
@@ -673,20 +749,19 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('propagates an error from a noop error boundary during synchronous mounting', () => {
-    const ops = [];
     class RethrowErrorBoundary extends React.Component {
       componentDidCatch(error) {
-        ops.push('RethrowErrorBoundary componentDidCatch');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary componentDidCatch');
         throw error;
       }
       render() {
-        ops.push('RethrowErrorBoundary render');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary render');
         return this.props.children;
       }
     }
 
-    function BrokenRender() {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
@@ -699,30 +774,34 @@ describe('ReactIncrementalErrorHandling', () => {
         );
       });
     }).toThrow('Hello');
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'RethrowErrorBoundary render',
       'BrokenRender',
-      // React doesn't retry because we're already rendering synchronously.
+
+      // React retries one more time
+      'RethrowErrorBoundary render',
+      'BrokenRender',
+
+      // Errored again on retry. Now handle it.
       'RethrowErrorBoundary componentDidCatch',
     ]);
     expect(ReactNoop.getChildren()).toEqual([]);
   });
 
   it('propagates an error from a noop error boundary during batched mounting', () => {
-    const ops = [];
     class RethrowErrorBoundary extends React.Component {
       componentDidCatch(error) {
-        ops.push('RethrowErrorBoundary componentDidCatch');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary componentDidCatch');
         throw error;
       }
       render() {
-        ops.push('RethrowErrorBoundary render');
+        Scheduler.unstable_yieldValue('RethrowErrorBoundary render');
         return this.props.children;
       }
     }
 
-    function BrokenRender() {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
@@ -738,10 +817,15 @@ describe('ReactIncrementalErrorHandling', () => {
         );
       });
     }).toThrow('Hello');
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'RethrowErrorBoundary render',
       'BrokenRender',
-      // React doesn't retry because we're already rendering synchronously.
+
+      // React retries one more time
+      'RethrowErrorBoundary render',
+      'BrokenRender',
+
+      // Errored again on retry. Now handle it.
       'RethrowErrorBoundary componentDidCatch',
     ]);
     expect(ReactNoop.getChildren()).toEqual([]);
@@ -794,15 +878,13 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('can schedule updates after uncaught error in render on mount', () => {
-    let ops = [];
-
-    function BrokenRender() {
-      ops.push('BrokenRender');
+    function BrokenRender({unused}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
       throw new Error('Hello');
     }
 
-    function Foo() {
-      ops.push('Foo');
+    function Foo({unused}) {
+      Scheduler.unstable_yieldValue('Foo');
       return null;
     }
 
@@ -810,58 +892,49 @@ describe('ReactIncrementalErrorHandling', () => {
     expect(() => {
       expect(Scheduler).toFlushWithoutYielding();
     }).toThrow('Hello');
-    expect(ops).toEqual([
+    ReactNoop.render(<Foo />);
+    expect(Scheduler).toHaveYielded([
       'BrokenRender',
       // React retries one more time
       'BrokenRender',
       // Errored again on retry
     ]);
-    ops = [];
-    ReactNoop.render(<Foo />);
-    expect(Scheduler).toFlushWithoutYielding();
-    expect(ops).toEqual(['Foo']);
+    expect(Scheduler).toFlushAndYield(['Foo']);
   });
 
   it('can schedule updates after uncaught error in render on update', () => {
-    let ops = [];
-
-    function BrokenRender(props) {
-      ops.push('BrokenRender');
-      if (props.throw) {
+    function BrokenRender({shouldThrow}) {
+      Scheduler.unstable_yieldValue('BrokenRender');
+      if (shouldThrow) {
         throw new Error('Hello');
       }
       return null;
     }
 
-    function Foo() {
-      ops.push('Foo');
+    function Foo({unused}) {
+      Scheduler.unstable_yieldValue('Foo');
       return null;
     }
 
-    ReactNoop.render(<BrokenRender throw={false} />);
-    expect(Scheduler).toFlushWithoutYielding();
-    ops = [];
+    ReactNoop.render(<BrokenRender shouldThrow={false} />);
+    expect(Scheduler).toFlushAndYield(['BrokenRender']);
 
     expect(() => {
-      ReactNoop.render(<BrokenRender throw={true} />);
+      ReactNoop.render(<BrokenRender shouldThrow={true} />);
       expect(Scheduler).toFlushWithoutYielding();
     }).toThrow('Hello');
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'BrokenRender',
       // React retries one more time
       'BrokenRender',
       // Errored again on retry
     ]);
 
-    ops = [];
     ReactNoop.render(<Foo />);
-    expect(Scheduler).toFlushWithoutYielding();
-    expect(ops).toEqual(['Foo']);
+    expect(Scheduler).toFlushAndYield(['Foo']);
   });
 
   it('can schedule updates after uncaught error during umounting', () => {
-    let ops = [];
-
     class BrokenComponentWillUnmount extends React.Component {
       render() {
         return <div />;
@@ -872,7 +945,7 @@ describe('ReactIncrementalErrorHandling', () => {
     }
 
     function Foo() {
-      ops.push('Foo');
+      Scheduler.unstable_yieldValue('Foo');
       return null;
     }
 
@@ -884,12 +957,11 @@ describe('ReactIncrementalErrorHandling', () => {
       expect(Scheduler).toFlushWithoutYielding();
     }).toThrow('Hello');
 
-    ops = [];
     ReactNoop.render(<Foo />);
-    expect(Scheduler).toFlushWithoutYielding();
-    expect(ops).toEqual(['Foo']);
+    expect(Scheduler).toFlushAndYield(['Foo']);
   });
 
+  // @gate skipUnmountedBoundaries
   it('should not attempt to recover an unmounting error boundary', () => {
     class Parent extends React.Component {
       componentWillUnmount() {
@@ -921,12 +993,17 @@ describe('ReactIncrementalErrorHandling', () => {
 
     ReactNoop.render(<Parent />);
     expect(Scheduler).toFlushWithoutYielding();
-    ReactNoop.render(null);
-    expect(Scheduler).toFlushAndYield([
-      // Parent unmounts before the error is thrown.
-      'Parent componentWillUnmount',
-      'ThrowsOnUnmount componentWillUnmount',
-    ]);
+
+    // Because the error boundary is also unmounting,
+    // an error in ThrowsOnUnmount should be rethrown.
+    expect(() => {
+      ReactNoop.render(null);
+      expect(Scheduler).toFlushAndYield([
+        'Parent componentWillUnmount',
+        'ThrowsOnUnmount componentWillUnmount',
+      ]);
+    }).toThrow('unmount error');
+
     ReactNoop.render(<Parent />);
   });
 
@@ -1272,13 +1349,14 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('unmounts components with uncaught errors', () => {
-    const ops = [];
     let inst;
 
     class BrokenRenderAndUnmount extends React.Component {
       state = {fail: false};
       componentWillUnmount() {
-        ops.push('BrokenRenderAndUnmount componentWillUnmount');
+        Scheduler.unstable_yieldValue(
+          'BrokenRenderAndUnmount componentWillUnmount',
+        );
       }
       render() {
         inst = this;
@@ -1291,7 +1369,7 @@ describe('ReactIncrementalErrorHandling', () => {
 
     class Parent extends React.Component {
       componentWillUnmount() {
-        ops.push('Parent componentWillUnmount [!]');
+        Scheduler.unstable_yieldValue('Parent componentWillUnmount [!]');
         throw new Error('One does not simply unmount me.');
       }
       render() {
@@ -1313,7 +1391,7 @@ describe('ReactIncrementalErrorHandling', () => {
       expect(Scheduler).toFlushWithoutYielding();
     }).toThrowError('Hello.');
 
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       // Attempt to clean up.
       // Errors in parents shouldn't stop children from unmounting.
       'Parent componentWillUnmount [!]',
@@ -1324,11 +1402,9 @@ describe('ReactIncrementalErrorHandling', () => {
   });
 
   it('does not interrupt unmounting if detaching a ref throws', () => {
-    let ops = [];
-
     class Bar extends React.Component {
       componentWillUnmount() {
-        ops.push('Bar unmount');
+        Scheduler.unstable_yieldValue('Bar unmount');
       }
       render() {
         return <span prop="Bar" />;
@@ -1337,10 +1413,10 @@ describe('ReactIncrementalErrorHandling', () => {
 
     function barRef(inst) {
       if (inst === null) {
-        ops.push('barRef detach');
+        Scheduler.unstable_yieldValue('barRef detach');
         throw new Error('Detach error');
       }
-      ops.push('barRef attach');
+      Scheduler.unstable_yieldValue('barRef attach');
     }
 
     function Foo(props) {
@@ -1348,16 +1424,13 @@ describe('ReactIncrementalErrorHandling', () => {
     }
 
     ReactNoop.render(<Foo />);
-    expect(Scheduler).toFlushWithoutYielding();
-    expect(ops).toEqual(['barRef attach']);
+    expect(Scheduler).toFlushAndYield(['barRef attach']);
     expect(ReactNoop.getChildren()).toEqual([div(span('Bar'))]);
 
-    ops = [];
-
     // Unmount
     ReactNoop.render(<Foo hide={true} />);
     expect(Scheduler).toFlushAndThrow('Detach error');
-    expect(ops).toEqual([
+    expect(Scheduler).toHaveYielded([
       'barRef detach',
       // Bar should unmount even though its ref threw an error while detaching
       'Bar unmount',
@@ -1381,37 +1454,36 @@ describe('ReactIncrementalErrorHandling', () => {
   it('error boundaries capture non-errors', () => {
     spyOnProd(console, 'error');
     spyOnDev(console, 'error');
-    let ops = [];
 
     class ErrorBoundary extends React.Component {
       state = {error: null};
       componentDidCatch(error) {
         // Should not be called
-        ops.push('componentDidCatch');
+        Scheduler.unstable_yieldValue('componentDidCatch');
         this.setState({error});
       }
       render() {
         if (this.state.error) {
-          ops.push('ErrorBoundary (catch)');
+          Scheduler.unstable_yieldValue('ErrorBoundary (catch)');
           return (
             <span
               prop={`Caught an error: ${this.state.error.nonStandardMessage}`}
             />
           );
         }
-        ops.push('ErrorBoundary (try)');
+        Scheduler.unstable_yieldValue('ErrorBoundary (try)');
         return this.props.children;
       }
     }
 
-    function Indirection(props) {
-      ops.push('Indirection');
-      return props.children;
+    function Indirection({children}) {
+      Scheduler.unstable_yieldValue('Indirection');
+      return children;
     }
 
     const notAnError = {nonStandardMessage: 'oops'};
-    function BadRender() {
-      ops.push('BadRender');
+    function BadRender({unused}) {
+      Scheduler.unstable_yieldValue('BadRender');
       throw notAnError;
     }
 
@@ -1422,9 +1494,8 @@ describe('ReactIncrementalErrorHandling', () => {
         </Indirection>
       </ErrorBoundary>,
     );
-    expect(Scheduler).toFlushWithoutYielding();
 
-    expect(ops).toEqual([
+    expect(Scheduler).toFlushAndYield([
       'ErrorBoundary (try)',
       'Indirection',
       'BadRender',
@@ -1470,17 +1541,17 @@ describe('ReactIncrementalErrorHandling', () => {
       }
     }
 
-    function ErrorMessage(props) {
+    function ErrorMessage({error}) {
       Scheduler.unstable_yieldValue('ErrorMessage');
-      return <span prop={`Caught an error: ${props.error.message}`} />;
+      return <span prop={`Caught an error: ${error.message}`} />;
     }
 
-    function BadRenderSibling(props) {
+    function BadRenderSibling({unused}) {
       Scheduler.unstable_yieldValue('BadRenderSibling');
       return null;
     }
 
-    function BadRender() {
+    function BadRender({unused}) {
       Scheduler.unstable_yieldValue('throw');
       throw new Error('oops!');
     }
@@ -1518,7 +1589,7 @@ describe('ReactIncrementalErrorHandling', () => {
     // This test seems a bit contrived, but it's based on an actual regression
     // where we checked for the existence of didUpdate instead of didMount, and
     // didMount was not defined.
-    function BadRender() {
+    function BadRender({unused}) {
       Scheduler.unstable_yieldValue('throw');
       throw new Error('oops!');
     }
@@ -1589,10 +1660,8 @@ describe('ReactIncrementalErrorHandling', () => {
     expect(ReactNoop.getChildren()).toEqual([
       span(
         'Caught an error:\n' +
-          (__DEV__
-            ? '    in BrokenRender (at **)\n'
-            : '    in BrokenRender\n') +
-          (__DEV__ ? '    in ErrorBoundary (at **).' : '    in ErrorBoundary.'),
+          '    in BrokenRender (at **)\n' +
+          '    in ErrorBoundary (at **).',
       ),
     ]);
   });
@@ -1625,40 +1694,168 @@ describe('ReactIncrementalErrorHandling', () => {
     expect(ReactNoop.getChildren()).toEqual([span('Caught an error: Hello')]);
   });
 
-  it('handles error thrown inside getDerivedStateFromProps of a module-style context provider', () => {
-    function Provider() {
-      return {
-        getChildContext() {
-          return {foo: 'bar'};
-        },
-        render() {
-          return 'Hi';
-        },
-      };
-    }
-    Provider.childContextTypes = {
-      x: () => {},
-    };
-    Provider.getDerivedStateFromProps = () => {
-      throw new Error('Oops!');
+  it('provides component stack even if overriding prepareStackTrace', () => {
+    Error.prepareStackTrace = function(error, callsites) {
+      const stack = ['An error occurred:', error.message];
+      for (let i = 0; i < callsites.length; i++) {
+        const callsite = callsites[i];
+        stack.push(
+          '\t' + callsite.getFunctionName(),
+          '\t\tat ' + callsite.getFileName(),
+          '\t\ton line ' + callsite.getLineNumber(),
+        );
+      }
+
+      return stack.join('\n');
     };
 
-    ReactNoop.render(<Provider />);
-    expect(() => {
-      expect(Scheduler).toFlushAndThrow('Oops!');
-    }).toErrorDev([
-      'Warning: The <Provider /> component appears to be a function component that returns a class instance. ' +
-        'Change Provider to a class that extends React.Component instead. ' +
-        "If you can't use a class try assigning the prototype on the function as a workaround. " +
-        '`Provider.prototype = React.Component.prototype`. ' +
-        "Don't use an arrow function since it cannot be called with `new` by React.",
-      'Legacy context API has been detected within a strict-mode tree.\n\n' +
-        'The old API will be supported in all 16.x releases, but ' +
-        'applications using it should migrate to the new version.\n\n' +
-        'Please update the following components: Provider',
+    class ErrorBoundary extends React.Component {
+      state = {error: null, errorInfo: null};
+      componentDidCatch(error, errorInfo) {
+        this.setState({error, errorInfo});
+      }
+      render() {
+        if (this.state.errorInfo) {
+          Scheduler.unstable_yieldValue('render error message');
+          return (
+            <span
+              prop={`Caught an error:${normalizeCodeLocInfo(
+                this.state.errorInfo.componentStack,
+              )}.`}
+            />
+          );
+        }
+        return this.props.children;
+      }
+    }
+
+    function BrokenRender(props) {
+      throw new Error('Hello');
+    }
+
+    ReactNoop.render(
+      <ErrorBoundary>
+        <BrokenRender />
+      </ErrorBoundary>,
+    );
+    expect(Scheduler).toFlushAndYield(['render error message']);
+    Error.prepareStackTrace = undefined;
+
+    expect(ReactNoop.getChildren()).toEqual([
+      span(
+        'Caught an error:\n' +
+          '    in BrokenRender (at **)\n' +
+          '    in ErrorBoundary (at **).',
+      ),
     ]);
   });
 
+  if (!ReactFeatureFlags.disableModulePatternComponents) {
+    it('handles error thrown inside getDerivedStateFromProps of a module-style context provider', () => {
+      function Provider() {
+        return {
+          getChildContext() {
+            return {foo: 'bar'};
+          },
+          render() {
+            return 'Hi';
+          },
+        };
+      }
+      Provider.childContextTypes = {
+        x: () => {},
+      };
+      Provider.getDerivedStateFromProps = () => {
+        throw new Error('Oops!');
+      };
+
+      ReactNoop.render(<Provider />);
+      expect(() => {
+        expect(Scheduler).toFlushAndThrow('Oops!');
+      }).toErrorDev([
+        'Warning: The <Provider /> component appears to be a function component that returns a class instance. ' +
+          'Change Provider to a class that extends React.Component instead. ' +
+          "If you can't use a class try assigning the prototype on the function as a workaround. " +
+          '`Provider.prototype = React.Component.prototype`. ' +
+          "Don't use an arrow function since it cannot be called with `new` by React.",
+        'Legacy context API has been detected within a strict-mode tree.\n\n' +
+          'The old API will be supported in all 16.x releases, but ' +
+          'applications using it should migrate to the new version.\n\n' +
+          'Please update the following components: Provider',
+      ]);
+    });
+  }
+
+  it('uncaught errors should be discarded if the render is aborted', async () => {
+    const root = ReactNoop.createRoot();
+
+    function Oops({unused}) {
+      Scheduler.unstable_yieldValue('Oops');
+      throw Error('Oops');
+    }
+
+    await ReactNoop.act(async () => {
+      ReactNoop.discreteUpdates(() => {
+        root.render(<Oops />);
+      });
+      // Render past the component that throws, then yield.
+      expect(Scheduler).toFlushAndYieldThrough(['Oops']);
+      expect(root).toMatchRenderedOutput(null);
+      // Interleaved update. When the root completes, instead of throwing the
+      // error, it should try rendering again. This update will cause it to
+      // recover gracefully.
+      root.render('Everything is fine.');
+    });
+
+    // Should finish without throwing.
+    expect(root).toMatchRenderedOutput('Everything is fine.');
+  });
+
+  it('uncaught errors are discarded if the render is aborted, case 2', async () => {
+    const {useState} = React;
+    const root = ReactNoop.createRoot();
+
+    let setShouldThrow;
+    function Oops() {
+      const [shouldThrow, _setShouldThrow] = useState(false);
+      setShouldThrow = _setShouldThrow;
+      if (shouldThrow) {
+        throw Error('Oops');
+      }
+      return null;
+    }
+
+    function AllGood() {
+      Scheduler.unstable_yieldValue('Everything is fine.');
+      return 'Everything is fine.';
+    }
+
+    await ReactNoop.act(async () => {
+      root.render(<Oops />);
+    });
+
+    await ReactNoop.act(async () => {
+      // Schedule a high pri and a low pri update on the root.
+      ReactNoop.discreteUpdates(() => {
+        root.render(<Oops />);
+      });
+      root.render(<AllGood />);
+      // Render through just the high pri update. The low pri update remains on
+      // the queue.
+      expect(Scheduler).toFlushAndYieldThrough(['Everything is fine.']);
+
+      // Schedule a high pri update on a child that triggers an error.
+      // The root should capture this error. But since there's still a pending
+      // update on the root, the error should be suppressed.
+      ReactNoop.discreteUpdates(() => {
+        setShouldThrow(true);
+      });
+    });
+    // Should render the final state without throwing the error.
+    expect(Scheduler).toHaveYielded(['Everything is fine.']);
+    expect(root).toMatchRenderedOutput('Everything is fine.');
+  });
+
   if (global.__PERSISTENT__) {
     it('regression test: should fatal if error is thrown at the root', () => {
       const root = ReactNoop.createRoot();
