@@ -5,7 +5,5 @@
  * LICENSE file in the root directory of this source tree.
  */
 
-'use strict';
-
 // TODO: this is special because it gets imported during build.
-module.exports = '16.12.0';
+export default '17.0.0';
